@@ -21,20 +21,22 @@
 
 import hydra
 from omegaconf import DictConfig
-from pytorch_lightning import Trainer
 from pytorch_segmentation_models_trainer.predict import predict
 from pytorch_segmentation_models_trainer.train import train
+from pytorch_segmentation_models_trainer.build_mask import build_masks
 from pytorch_segmentation_models_trainer.config_utils import validate_config
 
 
 @hydra.main(config_path="conf")
-def main(cfg: DictConfig) -> Trainer:
+def main(cfg: DictConfig):
     if cfg.mode == 'train':
         return train(cfg)
     elif cfg.mode == 'predict':
         return predict(cfg)
     elif cfg.mode == 'validate-config':
         return validate_config(cfg)
+    elif cfg.mode == 'build-mask':
+        return build_masks(cfg)
     else:
         raise NotImplementedError
 
