@@ -19,6 +19,8 @@
  ****
 """
 import logging
+from pytorch_segmentation_models_trainer.custom_callbacks.training_callbacks import FrameFieldComputeWeightNormLossesCallback
+from pytorch_segmentation_models_trainer.model_loader.frame_field_model import FrameFieldSegmentationPLModel
 
 import hydra
 from hydra.utils import instantiate
@@ -47,6 +49,16 @@ def train(cfg: DictConfig):
     model = Model(cfg) if "pl_model" not in cfg else import_module_from_cfg(cfg.pl_model)(cfg)
     trainer_logger = instantiate(cfg.logger) if "logger" in cfg else True
     callback_list = [instantiate(i) for i in cfg.callbacks] if "callbacks" in cfg else []
+    if isinstance(model, FrameFieldSegmentationPLModel):
+        is_norm_loss_added = False
+        for callback in callback_list:
+            if isinstance(callback, FrameFieldComputeWeightNormLossesCallback):
+                is_norm_loss_added = True
+                break
+        if not is_norm_loss_added:
+            callback_list.append(
+                FrameFieldComputeWeightNormLossesCallback()
+            )
     trainer = Trainer(**cfg.pl_trainer, logger=trainer_logger, callbacks=callback_list)
     trainer.fit(model)
     return trainer
